changecom//

// A5Sol version of CEW from M. Horie
// - execute_Launch() modified by W. Gardner to enhance crash diagnosis
// - added cew_Return macro to provide pass/fail status return to caller

const string operator+ (const string &s, int iPar)
{
   const int START = 0, NUM_CHAR = 1;

   int i = iPar;
   string result = "", sign = "";

   if (i < 0)
   {
      sign = "-";
      i = i * -1;
   }
   else
   {
      sign = "";
   }

   do {
      result.insert(START, NUM_CHAR, char((i % 10) + '0'));
      i = i / 10;
   }
   while (i > 0);

   return s + sign + result;
}

void cew_New_Handler()
{
   cout << endl << "FAILURE - 'new' operation did not succeed" << endl;
   exit(1);
}

define(cew_Variables,
   `static int cew_Error_Count=0;
   enum cew_Mode_Type {cew_Interactive, cew_Temp_Interactive, cew_Interactive_On_Failure, cew_Batch};
   cew_Mode_Type cew_Mode=cew_Batch;
   static int cew_Test_Count=0;'
)

define(cew_Set_Mode,
   cew_Mode=$1;
   set_new_handler(cew_New_Handler);
)

define(cew_Start_Menu,
   `define(cew_Menu_Instance,' 
   cew_Display_Mode=1;
   do {
      if (cew_Display_Mode)
         cout << endl;
)

define(cew_Menu_Item,
      if (cew_Display_Mode) 
         cout << " $1: $2" << endl;
      else if (cew_Menu_Selection=='$1') {
         cew_Display_Mode=1;
         $3;
         break;
      }
)

define(cew_Stop_Menu,
      if (cew_Display_Mode) 
         cout << " q: Quit" << endl;
      else if (cew_Menu_Selection=='q') {
         cew_Exit_Test_Case=1;
         break;
      } else {
         cout << endl << "Invalid menu selection" << endl;
         cew_Display_Mode=1;
      }
      if (cew_Display_Mode) {
         cout << endl << "Enter menu selection: ";
         cin >> cew_Menu_Selection;
         cew_Display_Mode=0;
      }
   } while (1);
   `)'
)

define(cew_Start_Exception_Handler_Builder,
   `define(cew_Exception_Handlers_Instance,'
)

define(cew_Build_Handler,
   catch ($1) {
      cew_Unexpected_Exception=1;
      cew_Unexpected_Exception_Name="$1";
   }
)

define(cew_Stop_Exception_Handler_Builder,
   catch (...) {
      cew_Unexpected_Exception=1;
      cew_Unexpected_Exception_Name="...";
   }
   `)'
)

define(cew_Summary,
   cout << endl << "********** Summary **********" << endl;
   cout << "Total number of test cases=" << cew_Test_Count << endl;
   cout << "Total number of test cases in error=" << cew_Error_Count << endl;
)

define(cew_Return,
   if ( cew_Test_Count>0 && cew_Error_Count==0 ) return 0;	// passed all
   return 1;	// failure return
)

define(cew_Trace_Declarations,
   class cew_Trace_Class {

      public:
         cew_Trace_Class() {
            cew_Display_Mode=1;
            cew_Exit_Test_Case=0;
            cew_Expected_Expression="";
            cew_Line_Number=0;
            cew_Menu_Selection='q';
            cew_Trace="";
         }

         int get_Exit_Test_Case() {
            return cew_Exit_Test_Case;
         }

         void set_Exit_Test_Case(int new_Value_Par) {
            cew_Exit_Test_Case = new_Value_Par;
         }

         const string get_Expected_Expression() {
            return cew_Expected_Expression;
         }

         int get_Line_Number() {
            return cew_Line_Number;
         }

         int get_Mode() {
            return cew_Display_Mode;
         }

         const string get_Trace() {
            return cew_Trace;
         }

         void execute_Menu() {
            cew_Menu_Instance
         }

         virtual void execute_Trace()=0;

      protected:
         int cew_Display_Mode;
         int cew_Exit_Test_Case;
         string cew_Expected_Expression;
         int cew_Line_Number;
         char cew_Menu_Selection;
         string cew_Trace;
         $1;
   };

   class cew_Ntrace : public cew_Trace_Class {

      public:
         cew_Ntrace() {
            cew_Actual_Expression="";
            cew_Actual_Value = "";
            cew_Expected_Value = "";
         }

         const string displayActualExpression() {
            return cew_Actual_Expression;
         }

         const string displayActualValue() {
            return cew_Actual_Value;
         }

         const string displayExpectedValue() {
            return cew_Expected_Value;
         }

      protected:
         string cew_Actual_Expression;
         string cew_Actual_Value;
         string cew_Expected_Value;
   };

   class cew_Etrace : public cew_Trace_Class {
   };

   class cew_Trace_Launch {

      public:
         cew_Trace_Launch() {
            cew_Declaration_Failure=1;
            cew_Execute_Trace=1;
            cew_Expected_Exception=0;
            cew_Signal_Trouble=0;
            cew_Status=0;
            cew_Unexpected_Exception=0;
            cew_Unexpected_Exception_Name="";
            cew_Update_Summary=1;
         }

         void execute_Launch() {
            cew_Test_Count++;
            cew_Child_PID=fork();
            if (!cew_Child_PID) {
               this->execute_Trace();
            }
            wait(&cew_Status);

	    // WBG mods start here
	    
	    if ( WIFEXITED(cew_Status) ) {	// child did call exit()
		if ( WEXITSTATUS(cew_Status)==1 )
		{
	            // An error was reported during the test; increment error count
         	    cew_Error_Count++;
		    cout << " Check test " << cew_Test_Count << endl;
      		}
	    }
	    else {				// did not call exit; why?
		cew_Error_Count++;

		if (cew_Status & WCOREFLAG)
		    cout << endl << "FAILURE - core dumped" << endl;
	  	else
	            cout << endl << "FAILURE - infinite loop?" << endl;
		
                cout << " Check test " << cew_Test_Count << endl;
	    }

	    // end of WBG mods
	 }

         virtual void execute_Trace()=0;

         virtual void start()=0;

         virtual void stop()=0;

      protected:
         pid_t cew_Child_PID;
         int cew_Declaration_Failure;
         int cew_Execute_Trace;
         int cew_Expected_Exception;
         int cew_Signal_Trouble;
         int cew_Status;
         cew_Trace_Class *cew_Trace;
         int cew_Unexpected_Exception;
         string cew_Unexpected_Exception_Name;
         int cew_Update_Summary;
   };

   class cew_Ntrace_Launch : public cew_Trace_Launch {

      public:
         cew_Ntrace_Launch() {
         }

         void display_OK_Trace() {
            cout << endl << "NO FAILURE (Ncase) in test number " << cew_Test_Count << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Actual value=" << ((cew_Ntrace *)cew_Trace)->displayActualValue() << endl;
            cout << " Expected expression=" << cew_Trace->get_Expected_Expression() << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         void display_Trace() {
            cout << endl << "FAILURE (Ncase) in test number " << cew_Test_Count << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Actual value=" << ((cew_Ntrace *)cew_Trace)->displayActualValue() << endl;
            cout << " Expected value=" << ((cew_Ntrace *)cew_Trace)->displayExpectedValue() << endl;
            cout << " Actual expression=" << ((cew_Ntrace *)cew_Trace)->displayActualExpression() << endl;
            cout << " Expected expression=" << cew_Trace->get_Expected_Expression() << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         void display_Trace(string unexpected_Exception_Par) {
            cout << endl << "FAILURE (Ncase) in test number " << cew_Test_Count << endl;
            cout << " Unexpected exception " << unexpected_Exception_Par << " thrown" << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Actual expression=" << ((cew_Ntrace *)cew_Trace)->displayActualExpression() << endl;
            cout << " Expected expression=" << cew_Trace->get_Expected_Expression() << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         virtual void execute_Trace() {
            try {
               start();
               cew_Declaration_Failure=0;
               do {
                  try {
                     if (cew_Execute_Trace) {
                        // Execute test
                        cew_Execute_Trace=0;
                        cew_Trace->execute_Trace();
                     }

                     if (cew_Unexpected_Exception) {
                        // An unexpected exception has been thrown
                        if (cew_Update_Summary) {
                           cew_Update_Summary=0;
                           cew_Signal_Trouble=1;
                        }

                        // Describe the problem
                        if (cew_Mode==cew_Interactive_On_Failure)
                           cew_Mode=cew_Temp_Interactive;
                        display_Trace(cew_Unexpected_Exception_Name);
                        cew_Unexpected_Exception=0;
                     } else if (((cew_Ntrace *)cew_Trace)->displayActualValue()!=((cew_Ntrace *)cew_Trace)->displayExpectedValue()) {
                        // The actual output does not match expected output
                        if (cew_Update_Summary) {
                           cew_Update_Summary=0;
                           cew_Signal_Trouble=1;
                        }

                        // Describe the problem
                        if (cew_Mode==cew_Interactive_On_Failure)
                           cew_Mode=cew_Temp_Interactive;
                        display_Trace();
                     } else if (((cew_Mode==cew_Interactive) || (cew_Mode==cew_Temp_Interactive)) && (cew_Trace->get_Mode())) {
                        // No problem encountered
                        display_OK_Trace();
                        cew_Update_Summary=0;
                     }

                     if ((cew_Mode==cew_Batch) || (cew_Mode==cew_Interactive_On_Failure))
                        cew_Trace->set_Exit_Test_Case(1);
                     else if (!cew_Trace->get_Exit_Test_Case()) {
                        // Allow user to do additional testing
                        cew_Trace->execute_Menu();
                     }
                  }
                  cew_Exception_Handlers_Instance
               } while (!cew_Trace->get_Exit_Test_Case());
               stop();
            }
            cew_Exception_Handlers_Instance

            if (cew_Unexpected_Exception) {
               // An unexpected exception has been thrown
               cew_Signal_Trouble=1;

               // Describe the problem
               cout << endl << "FAILURE (Ncase) in test number " << cew_Test_Count << endl;
               cout << " Unexpected exception " << cew_Unexpected_Exception_Name << " thrown in constructor" << endl;
               cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
            }

            if (cew_Signal_Trouble)
               exit(1);

            exit(0);
         }
   };

   class cew_Etrace_Launch : public cew_Trace_Launch {

      public:
         cew_Etrace_Launch() {
         }

         void display_OK_Trace() {
            cout << endl << "NO FAILURE (Ecase) in test number " << cew_Test_Count << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Expected exception " << cew_Trace->get_Expected_Expression() << " was not thrown" << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         void display_Trace(string unexpected_Exception_Par) {
            cout << endl << "FAILURE (Ecase) in test number " << cew_Test_Count << endl;
            cout << " Unexpected exception " << unexpected_Exception_Par << " thrown" << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         void display_Trace() {
            cout << endl << "FAILURE (Ecase) in test number " << cew_Test_Count << endl;
            cout << " Initial test trace=" << cew_Trace->get_Trace() << endl;
            cout << " Expected exception " << cew_Trace->get_Expected_Expression() << " was not thrown" << endl;
            cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
         }

         virtual void execute_Trace() {
            try {
               start();
               cew_Declaration_Failure=0;
               do {
                  try {
                     if (cew_Execute_Trace) {
                        // Execute test
                        cew_Execute_Trace=0;
                        cew_Trace->execute_Trace();
                     }

                     if (cew_Unexpected_Exception) {
                        // Exception was expected after all
                        if (cew_Trace->get_Expected_Expression() == cew_Unexpected_Exception_Name) {
                           cew_Unexpected_Exception = 0;
                           cew_Expected_Exception = 1;
                        }
                     }

                     if (cew_Unexpected_Exception) {
                        // An unexpected exception has been thrown
                        if (cew_Update_Summary) {
                           cew_Update_Summary=0;
                           cew_Signal_Trouble=1;
                        }

                        // Describe the problem
                        if (cew_Mode==cew_Interactive_On_Failure)
                           cew_Mode=cew_Temp_Interactive;
                        display_Trace(cew_Unexpected_Exception_Name);
                        cew_Unexpected_Exception=0;
                     }

                     if (!cew_Expected_Exception) {
                        // The expected exception has not been thrown
                        if (cew_Update_Summary) {
                           cew_Update_Summary=0;
                           cew_Signal_Trouble=1;
                        }

                        // Describe the problem
                        if (cew_Mode==cew_Interactive_On_Failure)
                           cew_Mode=cew_Temp_Interactive;
                           display_Trace();
                     }

                     if (((cew_Mode==cew_Interactive) || (cew_Mode==cew_Temp_Interactive)) && (cew_Trace->get_Mode()) && (!cew_Unexpected_Exception) && (cew_Expected_Exception)) {
                        // No problem encountered
                        display_OK_Trace();
                        cew_Update_Summary=0;
                        cew_Expected_Exception=0;
                     }

                     // Allow user to do additional testing
                     if ((cew_Mode==cew_Batch) || (cew_Mode==cew_Interactive_On_Failure))
                        cew_Trace->set_Exit_Test_Case(1);
                     else if (!cew_Trace->get_Exit_Test_Case()) {
                        cew_Trace->execute_Menu();
                     }
                  }
                  cew_Exception_Handlers_Instance
               } while (!cew_Trace->get_Exit_Test_Case());
               stop();
            }
            cew_Exception_Handlers_Instance

            if (!cew_Expected_Exception && cew_Declaration_Failure) {
               // No exception has been thrown
               cew_Signal_Trouble=1;

               // Describe the problem
               cout << endl << "FAILURE (Ecase) in test number " << cew_Test_Count << endl;
               cout << " Expected exception " << cew_Trace->get_Expected_Expression() << " not thrown in constructor" << endl;
               cout << " Source script line number=" << cew_Trace->get_Line_Number() << endl;
            }

            if (cew_Signal_Trouble)
               exit(1);

            exit(0);
         }
   };
)

define(cew_Ncase,
   class cew_Ntrace$2 : public cew_Ntrace {

      public:
         cew_Ntrace$2() {
            cew_Expected_Expression="$6";
            cew_Line_Number=$1;
            cew_Trace="$4";
            cew_Actual_Expression="$5";
         }

         virtual void execute_Trace() {
            $4;
            cew_Actual_Value = cew_Actual_Value + $5;
            cew_Expected_Value = cew_Expected_Value + $6;
         }

      protected:
         $3;
   };

   class cew_Ntrace_Launch$2 : public cew_Ntrace_Launch {

      public:
         void start() {
            cew_Trace = new cew_Ntrace$2();
         }

         void stop() {
            delete cew_Trace;
         }
   };

   cew_Ntrace_Launch$2 cew_nt$2;
   cew_nt$2.execute_Launch();
)

define(cew_Ecase, 
   class cew_Etrace$2 : public cew_Etrace {

      public:
         cew_Etrace$2() {
            cew_Expected_Expression="$5";
            cew_Line_Number=$1;
            cew_Trace="$4";
         }

         virtual void execute_Trace() {
            $4;
         }

      protected:
         $3;
   };

   class cew_Etrace_Launch$2 : public cew_Etrace_Launch {

      public:
         void start() {
            cew_Trace = new cew_Etrace$2();
         }

         void stop() {
            delete cew_Trace;
         }
   };

   cew_Etrace_Launch$2 cew_et$2;
   cew_et$2.execute_Launch();
)

cew_Variables

ifdef(PASSONE,undefine("define"))

