/* cheque.cpp
written by: Brian Hammond
revisions: Bill Gardner
last modified: February 14, 2001*/

#include "cheque.h"
#include <iostream>
#include <cstring>

using namespace std;

static char *dollarSign = "$";

void setChequeSym( char *string )
{
	if (string == 0) dollarSign = "$";
	else dollarSign = string;
}


// this is the dollars and cents version
char *chequeAmt( char *buffer, int spaces, unsigned dollars, unsigned cents ) 
	throw (ChequeBufferOverflow)
{
	// ensure dollars and cents are ok
	if (cents > 99)		//*** was 100; bug found by Farhan
	{
		dollars += cents / 100;
		cents %= 100;
	}

	// call other function for dollars, saving 3 spaces for decimals;
	// we rely on the other function to throw the exception
	chequeAmt(buffer, spaces - 3, dollars);
	
	// add decimal places to buffer string
	int strPtr = spaces - 3;
	buffer[strPtr] = '.';
	buffer[++strPtr] = static_cast<char>((cents / 10) + '0');
	buffer[++strPtr] = static_cast<char>((cents % 10) + '0');
	buffer[++strPtr] = 0;
	
	return buffer;
}


// this is the dollars-only version
char *chequeAmt( char *buffer, int spaces, unsigned dollars ) throw (ChequeBufferOverflow)
{
	buffer[0] = 0; // start with an empty buffer

	// temporary strings
	char tempString[15];
	char dollarString[15];

	int room = 0;		// space needed
	int commaCount = 0; // commas needed
	int count = 0;			// -> last char stored (before null)

	// special case for zero dollar value
	if (dollars == 0)
	{
		tempString[0] = '0';
		tempString[1] = 0;
	}
	else
	{
		unsigned temp = dollars; // temporary dollar holder

		// get each digit from end of number and convert to a character
		// (number is getting stored with 1's digit on the left)
		while (temp > 0)
		{
			int digit = temp % 10;
			temp /= 10;
			tempString[count] = static_cast<char>(digit + '0');
			count++;
		}

		tempString[count] = 0; // null terminate string
		count--;
	}

	// get room needed for digits and dollar symbol
	room = strlen(tempString);
	room += strlen(dollarSign);
	
	// figure out how many commas are needed, and add to room needed
	commaCount = (strlen(tempString) - 1) / 3;
	room += commaCount;
	
	// if spaces needed exceeds space parameter
	if (room > spaces) throw ChequeBufferOverflow();


	// how many characters precede the first comma
	int firstChars = strlen(tempString) % 3;
	if (firstChars == 0) firstChars = 3;

	int count2 = 0;		// no. of chars moved from tempString
	int count3 = 0;		// -> next char to be stored in dollarString

	// move number to dollarString in correct digit order with commas inserted
	while ((count - count2) >= 0)
	{
		if (commaCount != 0 && (count3 % 4 - firstChars) == 0)
		{
			dollarString[count3] = ',';
			count3++;
			commaCount--;
		}
		dollarString[count3] = tempString[count - count2];
		count2++;
		count3++;
	}

	dollarString[count3] = 0; // null terminate string

	strcat(buffer, dollarSign); // insert dollar symbols at front of buffer

	// any extra spaces are filled with * after the symbol
	for (count = spaces - room; count > 0; count--)
	{
		strcat(buffer, "*");
	}

	strcat(buffer, dollarString); // append dollar amount to string buffer
		
	return buffer;
}
	


